;###################################################################################################################################
;#                                                                                                                                 #
;#    Copyright (C) 2012 Francois Rebaudo (1)                                                                                      #
;#                                                                                                                                 #
;#     This program is free: you can redistribute the code and/or modify it under the terms of the GNU General Public              #
;#     License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later        #
;#     version.                                                                                                                    #
;#                                                                                                                                 #
;#     This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied          #
;#     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.       #
;#                                                                                                                                 #
;#     You should have received a copy of the GNU General Public License along with this program.                                  #
;#     If not, see <http://www.gnu.org/licenses/>.                                                                                 #
;#                                                                                                                                 #
;#     Scientists please be aware that the fact that this program is released as Free Software does not excuse you from            #
;#     scientiﬁc propriety,which obligates you to give appropriate credit. If you write a scientiﬁc paper describing research      #
;#     that made substantive use of this program, it is your obligation as a scientist to mention the fashion in which this        #
;#     model was used, including the version number, with a citation to the literature, in the Methods section.                    #
;#                                                                                                                                 #
;#    (1) IRD, UR 072, CNRS, UPR 9034, LEGS 91198 Gif-sur-Yvette Cedex and Universite Paris-Sud 11, 91405 Orsay Cedex, France      #
;#    francois.rebaudo@ird.fr                                                                                                      #
;#    last modified DECEMBER 14th 2012                                                                                             #
;#                                                                                                                                 #
;###################################################################################################################################

;extensions [profiler] ;provide a detailed file of execution time

;---------------------------------------------------------------------------------------------------------------------##############
;---------------------------------------------------------------------------------------------------------------------##############
;   VARIABLES
;---------------------------------------------------------------------------------------------------------------------##############
;---------------------------------------------------------------------------------------------------------------------##############

globals [
  version              ;program version
  numHAB               ;number of distinct habitat types
  count_rep            ;number based on OS clock to use it as unique time identifier
  h_choice             ;for drawing habitat types in the GUI
  
  
  listNt1              ;list of number of individual at generation Nt+1
  alternative          ;for calcul speed optimization
]

patches-own [
  habitat_barrier      ;resistance for migration of individuals (the destination cell holds the resistance value)
  habitat_resource     ;carrying capacity for individuals
  habitat_type         ;classification of habitats according to natural selection pressure
  
  layer                ;buffer to load example files
  
  Nt1                  ;number of individual at generation Nt+1
  listIDbis            ;probability at each location for fitness
  allwi                ;a pondered list of all individuals for the creation of generation n+1
  sampled?             ;a boolean to see if paches has been allready sampled to allow sampling WOR
]

turtles-own [
  cap_move             ;dispersal capability
  markerN              ;neutral markers, typically microsat
  markerS              ;loci under selection
  generation           ;to identify individuals from genereation n and n+1
  wi                   ;fitness
  mymum                ;lineage information
  mydad                ;lineage information
  sample               ;for the virtual sampling sub-model
]

;---------------------------------------------------------------------------------------------------------------------##############
;---------------------------------------------------------------------------------------------------------------------##############
;   INITIALIZATION
;---------------------------------------------------------------------------------------------------------------------##############
;---------------------------------------------------------------------------------------------------------------------##############

to setup ;clear all from previous runs, set version number
  clear-all
  set version "v20.07"
  no-display
end

to load_input_files ;load barrier, resource and habitat type files. Return "Error..." if failed. 
  carefully [
    setup_habitat_type_from_file          ;Read habitat_type file and store it into habitat_type patches
    setup_habitat_barrier_from_file       ;Read habitat_barrier file and store it into habitat_barrier patches
    setup_habitat_resource_from_file      ;Read habitat_resource file and store it into habitat_resource patches
  ] [
    print (word "Error in input files: ")
  ]
  color_habitat_barrier    ;check barriers and color
  color_habitat_resource   ;check resources and color
  color_habitat_type       ;check habitat types and color
  display no-display       ;user has a default displaying of habitat types
  reset-ticks
end

to load_input_GUI ;check input files loading
  check_hand_input
  color_habitat_barrier
  color_habitat_resource
  color_habitat_type
  reset-ticks
end

to export_GUI ;save "GUI-made" input matrices into input files (so that users can use them later on), and load input files created
  ;xxx
  if file-exists? "my_habitat_type.txt"     [file-delete "my_habitat_type.txt"]
  if file-exists? "my_habitat_resource.txt" [file-delete "my_habitat_resource.txt"]
  if file-exists? "my_habitat_barrier.txt"  [file-delete "my_habitat_barrier.txt"]
  
  carefully [
    file-open (word wd "my_habitat_type.txt")
    ask patches [
      file-type pxcor 
      file-type "  " 
      file-type pycor 
      file-type "  " 
      file-print habitat_type
    ]
    file-close
  ] []
  carefully [
    file-open (word wd "my_habitat_resource.txt")
    ask patches [
      file-type pxcor 
      file-type "  " 
      file-type pycor 
      file-type "  " 
      file-print habitat_resource
    ]
    file-close
  ] []
  carefully [
    file-open (word wd "my_habitat_barrier.txt")
    ask patches [
      file-type pxcor 
      file-type "  " 
      file-type pycor 
      file-type "  " 
      file-print habitat_barrier
    ]
    file-close
  ] []
  set habitat_type_file "my_habitat_type.txt"          ;GUI-made input files have the prefix "my_"
  set habitat_resource_file "my_habitat_resource.txt"
  set habitat_barrier_file "my_habitat_barrier.txt"
end

to setup_create_turtles
  set numHAB max([habitat_type] of patches)               ;Count the number of habitat types
  
  ifelse from_fstat_file = false [  ;users can start the simulation with a FSTAT file
    ifelse everywhere = false [     ;false: simulation of biological invasion // true: settled population
    create-turtles nb_agent [       ;set individuals characteristics
      set cap_move 100
      setxy in_X in_Y
      
      let list_allele_n (list)
      repeat num_microsat [            ;for each neutral markers
        let my_oneAllele (10 * sd_H)   ;set the mean value of potential alleles
        
        let allele1n "NA"
        let allele2n "NA"
        
        ifelse biall? = false [        ;if biall? = false -> microsat, if biall = true -> SNP
          set allele1n sqrt ((round (random-normal my_oneAllele sd_H)) ^ 2) ;choose allele 1 in a normal distribution
          set allele2n sqrt ((round (random-normal my_oneAllele sd_H)) ^ 2) ;choose allele 2 in a normal distribution
        ][
          set allele1n sqrt (random 2) ;choose allele 1 in [0;1]
          set allele2n sqrt (random 2) ;choose allele 2 in [0;1]
        ]
        
        set list_allele_n fput allele1n list_allele_n ;store alleles in a list ["locus 1 allele 1","locus 1 allele 2", ... ]
        set list_allele_n fput allele2n list_allele_n
        set mymum "NA" ;no lineage for the first generation
        set mydad "NA"
      ]
      set markerN list_allele_n
    
      set markerS n-values (num_locus_per_habitat * numHAB * 2) [random 2] ;markers under selection: alleles are either 0 or 1 (random)
    ]
  ] [ ;case of a settled population
    let xx 0
    while [xx < world-width] [
      let yy 0
      while [yy < world-height] [
        create-turtles nb_agent [
          set cap_move 100
          setxy xx yy
          
          let list_allele_n (list)
          repeat num_microsat [            ;for each neutral markers
            let my_oneAllele (10 * sd_H)   ;set the mean value of potential alleles
            
            let allele1n "NA"
            let allele2n "NA"
            
            ifelse biall? = false [        ;if biall? = false -> microsat, if biall = true -> SNP
              set allele1n sqrt ((round (random-normal my_oneAllele sd_H)) ^ 2) ;choose allele 1 in a normal distribution
              set allele2n sqrt ((round (random-normal my_oneAllele sd_H)) ^ 2) ;choose allele 2 in a normal distribution
            ][
              set allele1n sqrt (random 2) ;choose allele 1 in [0;1]
              set allele2n sqrt (random 2) ;choose allele 2 in [0;1]
            ]  
               
            set list_allele_n fput allele1n list_allele_n ;store alleles in a list ["locus 1 allele 1","locus 1 allele 2", ... ]
            set list_allele_n fput allele2n list_allele_n
            set mymum "NA" ;no lineage for the first generation
            set mydad "NA"
          ]
          set markerN list_allele_n
          set markerS n-values (num_locus_per_habitat * numHAB * 2) [random 2] ;markers under selection: alleles are either 0 or 1 (random)
        ]
        set yy yy + 1
      ]
      set xx xx + 1
    ]
  ]
  ] [
    ;check and read a fstat file, and create individuals, please refer to http://www2.unil.ch/popgen/softwares/fstat.htm for more information
    carefully [
      file-open (word wd FSTAT_file) ;read first two lines: number of habitat types and number of locations for individuals
        let num_hab file-read
        let num_loc file-read
      file-close
      
      file-open (word wd FSTAT_file)
        let j_count 0
        while [not file-at-end?] [
          let current_read file-read-line
          if item 0 current_read != "l" and j_count != 0 [
            let mymarkerN (list)
            let i_count 0
            repeat num_loc [
              let al1 read-from-string substring current_read (2 + i_count) (5 + i_count)
              let al2 read-from-string substring current_read (5 + i_count) (8 + i_count)
              set mymarkerN lput al1 mymarkerN
              set mymarkerN lput al2 mymarkerN
              set i_count i_count + 7
            ]
            ;print mymarkerN
            create-turtles 1 [
              set cap_move 100
              move-to one-of patches with [habitat_type = read-from-string (item 0 current_read)] ;individuals are NOT georeferenced in FSTAT so that we localize them randomly in the corresponding habitat type
              set markerN mymarkerN
              set markerS n-values (num_locus_per_habitat * numHAB * 2) [random 2] ;FSTAT do NOT deal with loci under selection here, so that they are randomly choosen
              set mymum "NA"
              set mydad "NA"
            ]
          ]
          set j_count j_count + 1
        ]
      file-close
    ] [ print (word"no input file or problem within input file \"" (word wd FSTAT_file) "\" ") ]
  ]
  
  agent_output_memory_alt ;store init. individuals into a file
  set count_rep substring (remove "." (remove "-" (remove " " (remove ":" date-and-time)))) 0 9 ;To dispose of number which seperate files in case of parallele runs
end

;---------------------------------------------------------------------------------------------------------------------##############
;---------------------------------------------------------------------------------------------------------------------##############
;   HARD CODED EXAMPLES FOR SOFTWARE PRESENTATION
;---------------------------------------------------------------------------------------------------------------------##############
;---------------------------------------------------------------------------------------------------------------------##############

to load_input_example3 ;HARD CODED simulation example for boxes configuration
  no-display
  set SRS "WR"
  set biall? false
  set from_fstat_file false
  apply_world_size 20 5
  ask patches [set habitat_type 0]
  ask patches [set habitat_barrier 10]
  ask patches [set habitat_resource 100]
  let list_h_type (list 2 2 1 1 1 2 2 1 1 1 2 2 1 1 1 2 2 1 1 1 2 2 1 1 1 2 2 1 1 1 2 2 1 1 1 2 2 1 1 1 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 1 1 1 2 2 1 1 1 2 2 1 1 1 2 2 1 1 1 2 2 1 1 1 2 2 1 1 1 2 2 1 1 1 2 2 1 1 1)
  do_layers list_h_type
  ask patches [set habitat_type layer set layer 0]
  color_habitat_barrier
  color_habitat_resource
  color_habitat_type
  display no-display
  
  set nb_agent 100  set in_Y 4  set in_X 10  set everywhere false
  set num_microsat 10  set num_locus_per_habitat 1  set sd_H 1  set mutation_rateN 1.0E-4  set var_dom_degree_a 0.5  set proba_dispersion 0.5  set var_coef_selec_vs_xx 0.5
  set nb_move 1
  
  set CSV true  set GENELAND true  set ARLEQUIN true  set GENEPOP false  set FSTAT false  set STRUCTURE false
  set n_ind 25  set n_points 20  set sample_pop? true  set output_freq 100
  
  set land_management "none"
  set land_time_step 1000
  set r_growth 0.5
  set var_num_generations 100
  
  reset-ticks
  
  setup_create_turtles
  
  go_init
end

to load_input_example2 ;HARD CODED simulation example for blocs configuration
  no-display
  set SRS "WR"
  set biall? false
  set from_fstat_file false
  apply_world_size 20 5
  ask patches [set habitat_type 0]
  ask patches [set habitat_barrier 10]
  ask patches [set habitat_resource 100]
  let list_h_type (list 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2)
  do_layers list_h_type
  ask patches [set habitat_type layer set layer 0]
  color_habitat_barrier
  color_habitat_resource
  color_habitat_type
  display no-display
  
  set nb_agent 100  set in_Y 4  set in_X 10  set everywhere false
  set num_microsat 10  set num_locus_per_habitat 1  set sd_H 1  set mutation_rateN 1.0E-4  set var_dom_degree_a 0.5  set proba_dispersion 0.5  set var_coef_selec_vs_xx 0.5
  set nb_move 1
  
  set CSV true  set GENELAND true  set ARLEQUIN true  set GENEPOP false  set FSTAT false  set STRUCTURE false
  set n_ind 25  set n_points 20  set sample_pop? true  set output_freq 100
  
  set land_management "none"
  set land_time_step 1000
  set r_growth 0.5
  set var_num_generations 100
  
  reset-ticks
  
  setup_create_turtles
  
  go_init
end

to load_input_example1 ;HARD CODED simulation example for random configuration
  no-display
  set SRS "WR"
  set biall? false
  set from_fstat_file false
  apply_world_size 20 5
  ask patches [set habitat_type 0]
  ask patches [set habitat_barrier 10]
  ask patches [set habitat_resource 100]
  let list_h_type (list 2 1 1 2 1 1 1 2 1 1 2 1 2 2 1 2 1 2 1 2 1 2 1 2 1 2 1 2 1 1 2 1 2 1 2 1 2 1 2 2 1 1 2 1 2 2 1 1 2 1 1 2 1 1 2 2 1 2 1 2 2 1 2 1 1 2 2 2 1 2 1 1 1 2 2 2 1 2 2 2 1 1 2 2 1 1 2 2 1 1 1 2 2 1 2 1 2 1 2 2 )
  do_layers list_h_type
  ask patches [set habitat_type layer set layer 0] 
  color_habitat_barrier
  color_habitat_resource
  color_habitat_type
  display no-display
  
  set nb_agent 100  set in_Y 4  set in_X 10  set everywhere false
  set num_microsat 10  set num_locus_per_habitat 1  set sd_H 1  set mutation_rateN 1.0E-4  set var_dom_degree_a 0.5  set proba_dispersion 0.5  set var_coef_selec_vs_xx 0.5
  set nb_move 1
  
  set CSV true  set GENELAND true  set ARLEQUIN true  set GENEPOP false  set FSTAT false  set STRUCTURE false
  set n_ind 25  set n_points 20  set sample_pop? true  set output_freq 100
  
  set land_management "random"
  set land_time_step 1
  set r_growth 0.5
  set var_num_generations 100
  
  reset-ticks
  
  setup_create_turtles
  
  go_init
end

to load_input_example_neutral_marker ;another HARD CODED simulation example using random configuration with a user-defined number of generations (time steps)
  no-display
  set SRS "WR"
  set biall? false
  set from_fstat_file false
  apply_world_size 20 5
  ask patches [set habitat_type 0]
  ask patches [set habitat_barrier 10]
  ask patches [set habitat_resource 100]
  let list_h_type (list 2 1 1 2 1 1 1 2 1 1 2 1 2 2 1 2 1 2 1 2 1 2 1 2 1 2 1 2 1 1 2 1 2 1 2 1 2 1 2 2 1 1 2 1 2 2 1 1 2 1 1 2 1 1 2 2 1 2 1 2 2 1 2 1 1 2 2 2 1 2 1 1 1 2 2 2 1 2 2 2 1 1 2 2 1 1 2 2 1 1 1 2 2 1 2 1 2 1 2 2 )
  do_layers list_h_type
  ask patches [set habitat_type layer set layer 0]
  color_habitat_barrier
  color_habitat_resource
  color_habitat_type
  display no-display
  
  set nb_agent 100  set in_Y 4  set in_X 10  set everywhere false
  set num_microsat 10  set num_locus_per_habitat 1  set sd_H 1  set mutation_rateN 1.0E-4  set var_dom_degree_a 0.5  set proba_dispersion 0.5  set var_coef_selec_vs_xx 0.5
  set nb_move 1
  
  set CSV true  set GENELAND false  set ARLEQUIN false  set GENEPOP false  set FSTAT false  set STRUCTURE false
  set n_ind 25  set n_points 20  set sample_pop? true  set output_freq 100
  
  set land_management "random"
  set land_time_step 1
  set r_growth 0.5
  ;set var_num_generations 100 ;no time step specification here
  
  reset-ticks
  
  setup_create_turtles
  
  go_init
end

;---------------------------------------------------------------------------------------------------------------------##############
;---------------------------------------------------------------------------------------------------------------------##############
;   MAIN PROGRAM
;---------------------------------------------------------------------------------------------------------------------##############
;---------------------------------------------------------------------------------------------------------------------##############

to go_init
  ;profiler:start
    repeat var_num_generations [go]                 
  ;profiler:stop
  
  ;let myfile (word "__profiler_N" num_microsat "_" ticks "_" "version" "_" (remove "-" (remove " " (remove ":" date-and-time))) ".txt")
  ;carefully [file-delete myfile][]
  ;file-open word wd myfile
  ;file-print profiler:report
  ;file-close

  ;profiler:reset
end

to go
  tick
  
  ;(1) : DISPERSION 
  ;--------------------------------------------------------------------------------------------- 
  main_dispersion 
  
  ;(2) : REPRODUCTION 
  ;---------------------------------------------------------------------------------------------
  ask one-of turtles [ifelse (( read-from-string (remove")" (remove "(turtle" (word self))) > 10000000 ) or (ticks > 100) ) [set alternative 1] [set alternative 2] ]
  
  ;alternative 2 is quicker while the cumulative number of agents is bellow a threshold, but it does not work for long simulations depending on your available RAM. 
  ;this is because of a known NetLogo issue: "clear-all" clears en masse while "ask turtles [die]" clear turtles one after the other. See https://github.com/NetLogo/NetLogo/issues/148 for more details
  
  ifelse alternative = 1 [
    main_reproduction
    main_kill
    main_new_generation
  ] [
    main_reproduction_alternative
  ]
  ;(3) SAMPLING
  ;---------------------------------------------------------------------------------------------
  if ticks mod output_freq = 0 [main_sampling]
  
  ;(4) OUTPUT FILES
  ;---------------------------------------------------------------------------------------------
  main_outputs
  if alternative = 2 [ main_kill_alternative ]
  
  ;(5) LANDSCAPE MANAGEMENT
  ;---------------------------------------------------------------------------------------------
  if ticks mod land_time_step = 0 [change_habitat_type land_management]
  
end
     ;----------------------------------------------------------------------------------------------------------------##############
     ;   MAIN PROGRAM SUB-MODELS 
     ;----------------------------------------------------------------------------------------------------------------##############
to main_kill_alternative
  ask turtles with [generation = "old"] [
    die
  ]
end
to main_kill
  clear-turtles
end
to main_outputs
  if (ticks mod output_freq = 0) [
    if GENEPOP   = TRUE [ifelse sample_pop? = true [agent_output_GENEPOP   1] [agent_output_GENEPOP   0]]
    if FSTAT     = TRUE [ifelse sample_pop? = true [agent_output_FSTAT     1] [agent_output_FSTAT     0]]
    if ARLEQUIN  = TRUE [ifelse sample_pop? = true [agent_output_ARLEQUIN  1] [agent_output_ARLEQUIN  0]]
    if STRUCTURE = TRUE [ifelse sample_pop? = true [agent_output_STRUCTURE 1] [agent_output_STRUCTURE 0]]
    if GENELAND  = TRUE [ifelse sample_pop? = true [agent_output_GENELAND  1] [agent_output_GENELAND  0]]
  
    if CSV       = TRUE [agent_output_memory_alt]
  ]
end
to main_sampling
  
  ask patches [set sampled? 0] ;all patches attribute sampled? is set to 0
  
  if (sample_pop? = True and ticks mod output_freq = 0) [
    ;print count(turtles with[sample = 1 and generation != "old"]) 
    ask turtles [set sample 0]
    ;type "time step = " print ticks
    let j 1
    while [j <= numHAB] [
      let n_points_count 0
      repeat n_points [
        ;carefully [ask one-of patches with [any? turtles-here with [sample = 0] and habitat_type = j ] [                    ;older version with WR patches
        carefully [ask one-of patches with [any? turtles-here with [sample = 0] and habitat_type = j and sampled? = 0] [     ;new version with WOR patches: in real-world you do not sampled twice in the same location
        
        ;# SCRIPT FOR VERIFICATION ##################################################
        ;let randomPatch nobody
        ;ask patches [type self type ";" type habitat_type print ";"]
        ;while [randomPatch = nobody] [
          ;set randomPatch one-of patches with [habitat_type = j]
          ;type "habitat_type = " type [habitat_type] of one-of patches type " ; j = " print j
        ;]
        ;print randomPatch
        ;carefully [
          ;let randomPatch one-of patches with [habitat_type = j]
          ;ask randomPatch [
        ;# END OF SCRIPT ############################################################
                  
          set sampled? 1
          let n_ind_count 0
          repeat n_ind [
            ifelse SRS = "WR" [
              carefully [ask one-of turtles-here with [mydad != nobody and generation != "old"] [set sample 1 set n_ind_count n_ind_count + 1]] [type "sampling error type 1.1"]  ;SRS WR = random sampling WITH replacement
            ] [
              carefully [ask one-of turtles-here with [mydad != nobody and generation != "old" and sample = 0] [set sample 1]] [type "sampling error type 1.2"]                   ;SRS WOR = random sampling WITHOUT replacement
            ]
          ]
          ;type "n_point = " type n_points_count type " ; " type "n_ind = " print n_ind_count
          set n_points_count n_points_count + 1
        ]] [type "sampling error at generation " type ticks type ": number of sampled locations is higher than the number of grid cells for habitat type " type j type ". Asked = " type n_points type ". Available = " type count patches with[habitat_type = j] print "."];if sampling fails
      ]
      set j j + 1
    ]
    ;print count(turtles with[sample = 1 and generation != "old"])
  ]  
end

to main_reproduction_alternative 
  ;set Nt1, the number of individuals at generation t+1, based on a logistic growth
  set listNt1 (list)
  ask patches [let N count(turtles-here) let r r_growth set Nt1 round( ( habitat_resource * N ) / ( N + ( habitat_resource - N ) * exp( - r * 1 ) ) )
    if Nt1 > 0 [set listNt1 fput (list pxcor pycor Nt1) listNt1]] ;logistic growth function
  
  ask turtles [ ;tag old generation to kill them at the end of the time step and set relative fitness (wi) in their location
    set generation "old"
    set wi 1
    
    let hab 0
    let allele 0
    while [hab < numHAB] [                                                    ;for each habitat
      let loc 0
      while [loc < (num_locus_per_habitat * 2)] [                             ;for each locus 
        let aa item (loc + allele * num_locus_per_habitat + 0) markerS
        let bb item (loc + allele * num_locus_per_habitat + 1) markerS
        ifelse habitat_type = hab + 1 [                                       ;if we are considering a locus associated with the habitat type the individual is in, then the relative fitness is (see Hartl, D.L., Clark, A.G. (2007) Principles of  population genetics (eds Sinauer Associates)):
          ifelse (aa = 1 and bb = 1) [                                        ;if genotype is "11"
            set wi wi * (1) ] [                                               
            ifelse ((aa = 1 and bb = 0) or (aa = 0 and bb = 1)) [             ;if genotype is "10" or "01" (which is the same for popoulation genetics)
              set wi wi * (1 - var_coef_selec_vs_xx * var_dom_degree_a)] [  
              ifelse (aa = 0 and bb = 0) [                                    ;if genotype is "00"
                set wi wi * (1 - var_coef_selec_vs_xx)] [
                print "Error !"]]]] [                                         ;else if we are considering a locus associated with another habitat that the one individual is on, then the relative fitness is:
          ifelse (aa = 1 and bb = 1) [                                        ;if genotype is "11"
            set wi wi * (1 - var_coef_selec_vs_xx) ] [
            ifelse ((aa = 1 and bb = 0) or (aa = 0 and bb = 1)) [             ;if genotype is "10" or "01"
              set wi wi * (1 - var_coef_selec_vs_xx * var_dom_degree_a) ] [
              ifelse (aa = 0 and bb = 0) [                                    ;if genotype is "00"
                set wi wi * (1) ] [ print "Error !" ]]]]
        set loc loc + 2 ]
      set hab hab + 1
      set allele allele + 2 ]]
  
  ;now creating the gamete pool, made of x repetitions of each turtle depending on its relative fitness wi. Briefly: 
  ;  if two turtles A and B, with wiA=1 and wiB=2, then gamte pool = [A, B, B], and then next turtles generation is randomly choosen so that
  ;  turtle B has twice much "chance" than turtle A to reproduce (2/3 and 1/3, respectively) => natural selection pressure based on adequation between genotype and habitat type
  ask patches with [any? turtles-here][                                       
    let allwi_bis (list [])
    set allwi_bis lput ([self] of turtles-here) allwi_bis
    foreach item 1 allwi_bis [set allwi_bis replace-item 0 allwi_bis (lput [wi] of ? (item 0 allwi_bis)) ]
    let x sum(item 0 allwi_bis)
    let mypercentage (max([habitat_resource] of patches) * 10) ;1000
    let allwinorm (list)
    foreach item 0 allwi_bis [
      let mywinorm precision ((? / x) * (mypercentage)) 0
      set allwinorm lput mywinorm allwinorm ]
    
    let allwiID item 1 allwi_bis
    let listID (list) ;list of id_agent with x repetitions per id_agent to represent wi = f(mypercentage)
    let i 0
    while [i < length(allwiID)]
    [
      set listID fput (n-values (item i allwinorm) [item i allwiID]) listID
      set i i + 1
    ]
    set listIDbis (list)
    foreach listID [foreach ? [set listIDbis lput ? listIDbis]]
  ]

  ;creation of the new generation
  foreach listNt1 [
    ;let wi xxx
    ifelse ([length(listIDbis)] of patch item 0 ? item 1 ?) > 0 [
      
      repeat item 2 ? [
        create-turtles 1 [
          setxy item 0 ? item 1 ?
          set cap_move 100
          set mymum one-of listIDbis
          set mydad one-of listIDbis
          set wi 1
          set sample 0
          set markerN (list)
          set markerS (list)
          
          let markerNF (list)
          ask mymum [set markerNF markerN]
          let markerNM (list)
          ask mydad [set markerNM markerN]
          let n 0
          while [n < (num_microsat * 2)] [
            ifelse random-float 1 < mutation_rateN [                        ;mutation rate for first allele
              set markerN lput (item (n + random 2) markerNF + 1) markerN
            ] [
              set markerN lput (item (n + random 2) markerNF) markerN
            ]
            ifelse random-float 1 < mutation_rateN [                        ;mutation rate for second allele
              set markerN lput (item (n + random 2) markerNM + 1) markerN
            ] [
              set markerN lput (item (n + random 2) markerNM) markerN
           ]
           set n n + 2
         ]
         
         let markerSF (list)
         ask mymum [set markerSF markerS]
         let markerSM (list)
         ask mydad [set markerSM markerS]
         let s 0
         while [s < (num_locus_per_habitat * numHAB * 2)] [
           set markerS lput (item (s + random 2) markerSF) markerS
           set markerS lput (item (s + random 2) markerSM) markerS
           set s s + 2
          ]
        ]
      ]
    
    ] [] 
  ]
end


to main_reproduction ;see https://github.com/NetLogo/NetLogo/issues/148 for more information. This part of the code is mostly COPY-PASTE of main_reproduction_alternative justified by the issue mentionned at this URL.
  ;type "-0-"
  ;set Nt1, the number of individuals at generation t+1, based on a logistic growth
  set listNt1 (list)
  ask patches [
    let N count(turtles-here)
    let r r_growth
    set Nt1 round( ( habitat_resource * N ) / ( N + ( habitat_resource - N ) * exp( - r * 1 ) ) )
    if Nt1 > 0 [
      set listNt1 fput (list pxcor pycor Nt1) listNt1
    ]
  ]
  ;type "-1-"
  ;set wi in agents' location: SAME AS ALTERNATIVE, see above for justification
  ask turtles [
    set generation "old"
    set wi 1
    
    let hab 0
    let allele 0
    while [hab < numHAB] [
      let loc 0
      while [loc < (num_locus_per_habitat * 2)] [
        let aa item (loc + allele * num_locus_per_habitat + 0) markerS
        let bb item (loc + allele * num_locus_per_habitat + 1) markerS
        ifelse habitat_type = hab + 1 [
          ifelse (aa = 1 and bb = 1) [
            set wi wi * (1) ] [
            ifelse ((aa = 1 and bb = 0) or (aa = 0 and bb = 1)) [
              set wi wi * (1 - var_coef_selec_vs_xx * var_dom_degree_a) ] [
              ifelse (aa = 0 and bb = 0) [
                set wi wi * (1 - var_coef_selec_vs_xx) ] [
                print "Error !"]]]] [
          ifelse (aa = 1 and bb = 1) [
            set wi wi * (1 - var_coef_selec_vs_xx) ] [
            ifelse ((aa = 1 and bb = 0) or (aa = 0 and bb = 1)) [
              set wi wi * (1 - var_coef_selec_vs_xx * var_dom_degree_a) ] [
              ifelse (aa = 0 and bb = 0) [
                set wi wi * (1) ] [
                print "Error !"]]]]
        set loc loc + 2
      ]
      set hab hab + 1
      set allele allele + 2
    ]
  ]
  ;type "-2-"
  
  ask patches with [any? turtles-here][
    set allwi (list [][][])                                                 ;here we use allwi because all turtles will be cleared at the end of this procedure. allwi = [wi][markerN][markerS][turtle]
    set allwi lput ([self] of turtles-here) allwi
    foreach item 3 allwi [
      set allwi replace-item 0 allwi (lput [wi] of ? (item 0 allwi)) 
      set allwi replace-item 1 allwi (lput [markerN] of ? (item 1 allwi)) 
      set allwi replace-item 2 allwi (lput [markerS] of ? (item 2 allwi)) 
    ]

    let x sum(item 0 allwi)
    let mypercentage (max([habitat_resource] of patches) * 10) ;1000
    
    let allwinorm (list)
    foreach item 0 allwi [
      let mywinorm precision ((? / x) * (mypercentage)) 0
      set allwinorm lput mywinorm allwinorm 
    ]
    let allwiID item 3 allwi
    
    let listID (list) ;list of id_agent with x repetitions per id_agent to represent wi = f(mypercentage)
    let i 0
    while [i < length(allwiID)] [
      set listID fput (n-values (item i allwinorm) [item i allwiID]) listID
      set i i + 1
    ]
    set listIDbis (list)
    foreach listID [foreach ? [set listIDbis lput (word ?) listIDbis]]
  ]
  ;type "-3-"
  
  ask patches with [any? turtles-here][ 
    let lgallwi 0
    while [lgallwi < length(item 3 allwi)] [
      set allwi replace-item 3 allwi (replace-item lgallwi (item 3 allwi) (( word item lgallwi (item 3 allwi) )))
      set lgallwi lgallwi + 1
    ]
  ]
  ;type "-4-"
end

  
  
  
to main_new_generation
  ;creation of the new generation
  foreach listNt1 [ ;listNt1 contains the number of turtles for next generation at each localization (x,y): listNt1 = [x][y][Nt1]
    ;let wi xxx
    ifelse ([length(listIDbis)] of patch item 0 ? item 1 ?) > 0 [
      repeat item 2 ? [ ;number of turtles at generation t1
        create-turtles 1 [
          setxy item 0 ? item 1 ? ;(X;Y)
          set cap_move 100
          set mymum one-of listIDbis
          set mydad one-of listIDbis
          
          set wi 1
          set sample 0
          set markerN (list)
          set markerS (list)
          
          let markerNF (list)
          let markerNM (list)
          
          let i_count 0
          let count_mymum "na"
          let count_mydad "na"
          foreach [item 3 allwi] of patch-here [                                 ;allwi = [wi][markerN][markerS][turtle]; item 3 = turtle
            ;print ?
            if mymum = ? [set count_mymum i_count]
            if mydad = ? [set count_mydad i_count]
            set i_count i_count + 1
          ]
          set mymum (word mymum)
          set mydad (word mydad)
          
          set markerNF (item count_mymum ([item 1 allwi] of patch-here))
          set markerNM (item count_mydad ([item 1 allwi] of patch-here))

          let n 0
          while [n < (num_microsat * 2)] [
            ifelse random-float 1 < mutation_rateN [                             ;mutation rate for first allele
              set markerN lput (item (n + random 2) markerNF + 1) markerN
            ] [
              set markerN lput (item (n + random 2) markerNF) markerN
            ]
            ifelse random-float 1 < mutation_rateN [                             ;mutation rate for second allele
              set markerN lput (item (n + random 2) markerNM + 1) markerN
            ] [
              set markerN lput (item (n + random 2) markerNM) markerN
           ]
           set n n + 2
         ]
         
         let markerSF (list)
         let markerSM (list)
         set markerSF (item count_mymum ([item 2 allwi] of patch-here))
         set markerSM (item count_mydad ([item 2 allwi] of patch-here))
         let s 0
         while [s < (num_locus_per_habitat * numHAB * 2)] [
           set markerS lput (item (s + random 2) markerSF) markerS
           set markerS lput (item (s + random 2) markerSM) markerS
           set s s + 2
          ]
        ]
      ]
    ] [print "listIDbis"] 
  ]
end

to main_dispersion
    ask turtles [ 
    if random-float 1 < proba_dispersion [                                                ;if migration rate
      repeat nb_move [                                                                    ;for a given number of movements
        let destination one-of neighbors with [habitat_barrier < [cap_move] of myself]    ;for selected destinations
        if is-patch? destination [
          move-to destination                                                             ;move
          set cap_move cap_move - [habitat_barrier] of patch-here                         ;reduce dispersal capabilities
        ] 
      ]
    ]
  ]
end

     ;----------------------------------------------------------------------------------------------------------------##############
     ;   INITIALIZATION SUB-MODELS
     ;----------------------------------------------------------------------------------------------------------------##############
to do_layers [list_values] ;copy hard-coded inputs into patches
  let x 0
  let count_i 0
  while [x < 20] [
    let y 0
    while [y < 5] [
      ask patch x y [set layer item count_i list_values]
      set y y + 1
      set count_i count_i + 1
    ]
    set x x + 1
  ]
end

to check_hand_input ;check input files
  ifelse count(patches with [habitat_type = 0]) = (world-height * world-width) [
    ask patches [
      set habitat_type random 2 + 1
      set habitat_barrier 10
      set habitat_resource 10
    ]
    print "No input files have been defined, and default values attributed to the current simulation (resources = 10, barrier = 10, habitat types = 2)"
  ] [
    ask patches with [habitat_type = 0] [ print (word "Patch (" pxcor "; " pycor ") has not been initialized !") ] 
  ]
  if count(patches with [habitat_type = 0]) = 0 [print "Your landscape has been correctly initialized"]
end     
to setup_habitat_barrier_from_file
  carefully
  [
    file-open (word wd habitat_barrier_file)
    while [not file-at-end?]
    [
      let next-X file-read
      let next-Y file-read
      let next-habitat_barrier file-read
      carefully [ask patch next-X next-Y [set habitat_barrier next-habitat_barrier]] []
    ]
    file-close
  ] [
    print (word "no input file \"" (word wd habitat_barrier_file) "\": default value of 10 attributed to all patches")
    ask patches [set habitat_barrier 10]
  ]
end

to setup_habitat_resource_from_file  
  carefully
  [
    file-open (word wd habitat_resource_file)
    while [not file-at-end?]
    [
      let next-X file-read
      let next-Y file-read
      let next-habitat_resource file-read
      carefully [ask patch next-X next-Y [set habitat_resource next-habitat_resource]] []
    ]
    file-close
  ] [
    print (word "no input file \"" (word wd habitat_resource_file) "\": default value of 10 attributed to all patches")
    ask patches [set habitat_resource 10]
  ]
end

to setup_habitat_type_from_file
  carefully
  [
    file-open (word wd habitat_type_file)
    let next-X 0
    let next-Y 0
    let max-X 0
    let max-Y 0
    while [not file-at-end?]
    [
      set next-X file-read 
      if next-X > max-X [set max-X next-X]
      set next-Y file-read 
      if next-Y > max-Y [set max-Y next-Y]
      let next-habitat_type file-read
      ;carefully [ask patch next-X next-Y [set habitat_type next-habitat_type]] []
    ]
    file-close
    
    apply_world_size max-X + 1 max-Y + 1
    
    file-open (word wd habitat_type_file)
    set next-X 0
    set next-Y 0
    while [not file-at-end?]
    [
      set next-X file-read 
      set next-Y file-read 
      let next-habitat_type file-read
      carefully [ask patch next-X next-Y [set habitat_type next-habitat_type]] []
    ]
    file-close
    
  ] [
    ;print "The landscape scale have not been initialized beacuse of an error in your input files. Please check your input files and try again."
    print (word "no input file \"" (word wd habitat_type_file) "\": default value of 2 habitat types randomly distributed among patches")
    ask patches [set habitat_type random 2 + 1]
  ]
end

to apply_world_size [world_length world_height]
  ; set the width and height of the landscape in number of patches as defined by the GUI
  ; with INPUT "world_height" and INPUT "world_length"
  ; Error handling = print text and default values
  carefully [
    resize-world 0 (world_length - 1) 0 (world_height - 1)
  ] [
    print (word "Error in input numbers \"world_height\" (=" world_height " and \"world_length\" (="world_length" ! Default values are 5 and 20.")
    resize-world 0 19 0 4
  ]
end



     ;----------------------------------------------------------------------------------------------------------------##############
     ;   MAIN PROGRAM SUB-MODELS (OUTPUTS AND REPORTERS)
     ;----------------------------------------------------------------------------------------------------------------##############

to agent_output_memory_alt ;store info in a file
    let numtick "" ;We wanted output files to be sorted automatically so that numtick is formatted on three-digit numbers from 001 to 999
    ifelse ticks < 10 [set numtick (word "00" ticks)] [ifelse ticks < 100 [set numtick (word "0" ticks)] [set numtick ticks]]
    carefully [file-delete (word "outputAgentMemory_" numtick "_" (remove "-" (remove " " (remove ":" date-and-time))) ".csv")][]
    file-open (word wd "outputAgentMemory_" numtick "_" remove "-" (remove " " (remove ":" date-and-time)) "_" behaviorspace-run-number ".csv")
    
    ask turtles with [mydad != nobody] [
      file-type self file-type ";" file-type cap_move file-type ";" file-type 1 file-type ";" (foreach markerN [file-type ? file-type ";"]) (foreach markerS [file-type ? file-type ";"]) file-type mymum file-type ";" file-type mydad file-type ";" file-type [pxcor] of patch-here file-type ";" file-type [pycor] of patch-here file-type ";" file-type [habitat_type] of patch-here file-print ""
    ]
    
    file-close
end

to agent_output_GENELAND [sample_number] ;according to http://www2.imm.dtu.dk/~gigu/Geneland/Geneland-Doc.pdf
  ;(A) file name
  let numtick ""
  ifelse ticks < 10 [set numtick (word "00" ticks)] [ifelse ticks < 100 [set numtick (word "0" ticks)] [set numtick ticks]]
  let myfile1 (word "input_geneland_" numtick "_" count_rep "_" behaviorspace-run-number ".markers")
  carefully [file-delete myfile1][]
  let myfile2 (word "input_geneland_" numtick "_" count_rep "_" behaviorspace-run-number ".coordinates")
  carefully [file-delete myfile2][]
  
  
  ;file-open word wd myfile1
  ifelse sample_number = 0 [
    ask turtles with [mydad != nobody] [
      
      file-open word wd myfile1
      foreach markerN [file-type ? file-type " "]
      file-print ""
      file-close
      
      file-open word wd myfile2
      file-print (word xcor " " ycor)
      file-close
    ]
  ] [
    ask turtles with [mydad != nobody and sample = 1] [
      
      file-open word wd myfile1
      foreach markerN [file-type ? file-type " "]
      file-print ""
      file-close
      
      file-open word wd myfile2
      file-print (word xcor " " ycor)
      file-close
      
    ]
  ]

end

;-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

to agent_output_FSTAT [sample_number] ;according to http://www2.unil.ch/popgen/softwares/fstat.htm 
  ;creating input with 1 population = 1 habitat type
  ;--------------------------------------------------
  let numtick ""
  ifelse ticks < 10 [set numtick (word "00" ticks)] [ifelse ticks < 100 [set numtick (word "0" ticks)] [set numtick ticks]]
  let myfile (word "input_fstat_hab_" numtick "_" count_rep "_" behaviorspace-run-number ".dat")
  let nbpop max([habitat_type] of patches with [any? turtles-here])
  let nbloci num_microsat
  let baseal 100
  let lenal 3
  let listloci ("loc1\nloc2\nloc3\nloc4\nloc5\nloc6\nloc7\nloc8\nloc9\nloc10")
  let lastX ""
  let lastY ""
  let incr 1
  
  file-open word wd myfile
  file-print (word nbpop " " nbloci " " baseal " " lenal)
  file-print listloci
    
  let habitat 1
  while [habitat <= nbpop] [
    
    ifelse sample_number = 0 [
    ask turtles with [mydad != nobody and habitat = [habitat_type] of patch-here] [
      let listal10al2 (list)

      let i 0
      while [i < (num_microsat * 2)] [
        let al1 item (i + 0) markerN
        let al2 item (i + 1) markerN
        ifelse (al1 < 10) [set al1 (word "00" al1)] [ifelse (al1 < 100) [set al1 (word "0" al1)] []]
        ifelse (al2 < 10) [set al2 (word "00" al2)] [ifelse (al2 < 100) [set al2 (word "0" al2)] []]
       
        set listal10al2 lput (word al1 al2) listal10al2
        set i i + 2
      ]
      file-print remove "]" (remove "[" (word habitat " " listal10al2))
      ;print remove "]" (remove "[" (word habitat " " listal10al2))
    ]
    ] [
    ask turtles with [mydad != nobody and habitat = [habitat_type] of patch-here and sample = 1] [
      let listal10al2 (list)

      let i 0
      while [i < (num_microsat * 2)] [
        let al1 item (i + 0) markerN
        let al2 item (i + 1) markerN
        ifelse (al1 < 10) [set al1 (word "00" al1)] [ifelse (al1 < 100) [set al1 (word "0" al1)] []]
        ifelse (al2 < 10) [set al2 (word "00" al2)] [ifelse (al2 < 100) [set al2 (word "0" al2)] []]
       
        set listal10al2 lput (word al1 al2) listal10al2
        set i i + 2
      ]
      file-print remove "]" (remove "[" (word habitat " " listal10al2))
      ;print remove "]" (remove "[" (word habitat " " listal10al2))
    ]    
    ]
    
    
    set habitat habitat + 1
  ]
  file-close
end

;-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

to agent_output_ARLEQUIN [sample_number] ;according to http://cmpg.unibe.ch/software/arlequin3/ 
  
  let numtick ""
  ifelse ticks < 10 [set numtick (word "00" ticks)] [ifelse ticks < 100 [set numtick (word "0" ticks)] [set numtick ticks]]
  let myfile (word "input_arlequin_hab_" numtick "_" count_rep "_" behaviorspace-run-number ".arp")
  carefully [file-delete myfile][]
  
  ;(A) comments regarding the model
  let comments (word "# SimAdapt output " version " for ARLEQUIN software V3.1 and V3.5" "\n" 
    "# Output date and time: " date-and-time "\n"
    "# mutation_rateN= " mutation_rateN "\n"
    "# proba_dispersion= " proba_dispersion "\n"
    "# r_growth= " r_growth "\n"
    "# nb_move= " nb_move "\n"
    "# var_coef_selec_vs_xx= " var_coef_selec_vs_xx "\n"
    "# var_dom_degree_x= " var_dom_degree_a "\n"
    "# var_num_generations= " ticks "\n"
    "# numHAB= " numHAB "\n")

  ;(B) DATA
  let data (word "[Data]" "\n")
  
  let lastX ""
  let lastY ""
  let SampleSize 0
  let SampleSizeCount 0
  let SampleName ""
  let SampleDataList (list)
  let SampleData ""
  let controlSampleSize 0
  let incr 0
  
  let habitat 1
  let num_habitat max([habitat_type] of patches with [any? turtles-here])
  
  
  while [habitat <= num_habitat] [
    set SampleName (word "\"hab" habitat "\"")
    
    
    
    ifelse sample_number = 0 [
    ask turtles with [mydad != nobody and habitat = [habitat_type] of patch-here] [
      set SampleSizeCount SampleSizeCount + 1
      let indname remove " " (word self "_" xcor "_" ycor "_" [habitat_type] of patch-here)

      set SampleData (word SampleData "\n" indname " 1" "       ")
      let i 0
      while [i < (num_microsat * 2)] [
        let allele item i (markerN)
        set SampleData (word SampleData allele " ")
        set i i + 2
      ]
      set SampleData (word SampleData "\n" "                  ")
      set i 1
      while [i < (num_microsat * 2)] [
        let allele item i (markerN)
        set SampleData (word SampleData allele " ")
        set i i + 2
      ]
      set incr incr + 1 
    ]
    ] [
    ask turtles with [mydad != nobody and habitat = [habitat_type] of patch-here and sample = 1] [
      set SampleSizeCount SampleSizeCount + 1
      let indname remove " " (word self "_" xcor "_" ycor "_" [habitat_type] of patch-here)

      set SampleData (word SampleData "\n" indname " 1" "       ")
      let i 0
      while [i < (num_microsat * 2)] [
        let allele item i (markerN)
        set SampleData (word SampleData allele " ")
        set i i + 2
      ]
      set SampleData (word SampleData "\n" "                  ")
      set i 1
      while [i < (num_microsat * 2)] [
        let allele item i (markerN)
        set SampleData (word SampleData allele " ")
        set i i + 2
      ]
      set incr incr + 1 
    ]
    ]
    
    set SampleSize SampleSizeCount
    
    let sampleX (word
      "    SampleName=" SampleName "\n"
      "    SampleSize=" SampleSize "\n"
      "    SampleData={"
      SampleData "\n"
      "}"
      )
      
    set SampleDataList fput sampleX SampleDataList
      
    set SampleSizeCount 0
    set SampleData ""
    
    set habitat habitat + 1
  ]
  
  ;(C) PROFILE
  let NbSamples count patches with [any? turtles-here]
  let profile (word "[Profile]" "\n" 
    "  Title=\"An analysis of SimAdapt outputs using ARLEQUIN V3.1 and V3.5\"" "\n"
    "  NbSamples=" controlSampleSize "\n"
    "  DataType=STANDARD" "\n"
    "  GenotypicData=1" "\n"
    "  GameticPhase=0" "\n"
    "  MissingData=\"?\"" "\n"
    "  LocusSeparator=WHITESPACE" "\n")
  
  ;(D) STRUCTURE
  let structureS (word "  [[Structure]]" "\n"
    "    StructureName=\"New Edited Structure\"" "\n"
    "    NbGroups="num_habitat"" "\n")
  let i 1
  while [i <= num_habitat] [
    set structureS (word structureS
    "    Group={" "\n"
    "      \"hab" i "\"" "\n"
    "    }" "\n")
    set i i + 1
  ]
  
  ;(E) writting in the file
  file-open word wd myfile
    file-print comments
    file-print profile
    file-print data
    file-print "  [[Samples]]"
    foreach SampleDataList
    [
      file-print ?
    ]
    file-print structureS
  file-close
  
end

;-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

to agent_output_STRUCTURE [sample_number] ;according to http://pritch.bsd.uchicago.edu/structure.html
  
  ;creating input with 1 population = 1 xy coordinate
  ;--------------------------------------------------
  ;(A) file name
  let numtick ""
  ifelse ticks < 10 [set numtick (word "00" ticks)] [ifelse ticks < 100 [set numtick (word "0" ticks)] [set numtick ticks]]
  let myfile (word "input_structure_" numtick "_" count_rep "_" behaviorspace-run-number )
  carefully [file-delete myfile][]

  file-open word wd myfile
  
  ifelse sample_number = 0 [
    ask turtles with [mydad != nobody] [
      let sLabel (word self "_" xcor "_" ycor "_" [habitat_type] of patch-here)
      set sLabel remove " " sLabel
      let sPopData (word "X" xcor "_Y" ycor "")
      let sPopFlag 1
      let sLocData [habitat_type] of patch-here
      let sGenotypeData markerN
    
      file-print remove "]" (remove "[" (word sLabel " " sPopData " " sPopFlag " " sLocData " " sGenotypeData))
    ]
  ] [
    ask turtles with [mydad != nobody and sample = 1] [
      let sLabel (word self "_" xcor "_" ycor "_" [habitat_type] of patch-here)
      set sLabel remove " " sLabel
      let sPopData (word "X" xcor "_Y" ycor "")
      let sPopFlag 1
      let sLocData [habitat_type] of patch-here
      let sGenotypeData markerN
    
      file-print remove "]" (remove "[" (word sLabel " " sPopData " " sPopFlag " " sLocData " " sGenotypeData))
    ]
  ]
    
   file-close
end

;-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

to agent_output_GENEPOP [sample_number]
  ;creating input with 1 population = 1 xy coordinate
  ;--------------------------------------------------
  ;(A) Title for the first line 
  let title (word "Title line: SimAdapt output " version " for GENEPOP software V4.1, output date and time: " date-and-time 
    " . mutation_rateN= " mutation_rateN
    " ; proba_dispersion= " proba_dispersion 
    " ; r_growth= " r_growth
    " ; nb_move= " nb_move 
    " ; var_coef_selec_vs_xx= " var_coef_selec_vs_xx 
    " ; var_dom_degree_x= " var_dom_degree_a 
    " ; var_num_generations= " ticks 
    " ; numHAB= " numHAB 
    )
  
  ;(B) list of locus names and population seperator 
  let list_locus "loc1, loc2, loc3, loc4, loc5, loc6, loc7, loc8, loc9, loc10"
  let pop_sep "Pop"
  
  ;(C) file name
  let numtick ""
  ifelse ticks < 10 [set numtick (word "00" ticks)] [ifelse ticks < 100 [set numtick (word "0" ticks)] [set numtick ticks]]
  let myfile (word "input_genepop_hab_" numtick "_" count_rep "_" behaviorspace-run-number )
  carefully [file-delete myfile][]
  
  ;(D) writting the file
  file-open word wd myfile
    file-print title
    file-print list_locus
    
    let habitat 1
    let num_habitat max([habitat_type] of patches with [any? turtles-here])
    while [habitat <= num_habitat] [
      file-print pop_sep
      
      
      
      ifelse sample_number = 0 [
      ask turtles with [mydad != nobody and habitat = [habitat_type] of patch-here] [
        let indname word self ""
        file-type (word (remove " " indname) "_" xcor "_" ycor "_" [habitat_type] of patch-here ",")
        
        let i 0
        while [i < (num_microsat * 2)] [
          let allele1 item (i + 0) (markerN)
          let allele2 item (i + 1) (markerN)
          let genotype "000000"
            
          ;allele 1
          ifelse allele1 < 10 [
            set allele1 word "00" allele1
          ] [
            if allele1 < 100 [
              set allele1 word "0" allele1
            ]
          ]
          
          ;allele 2
          ifelse allele2 < 10 [
            set allele2 word "00" allele2
          ] [
            if allele2 < 100 [
              set allele2 word "0" allele2
            ]
          ]
          
          ;genotype
          set genotype word allele1 allele2
            
          file-type word genotype " "
            
          set i i + 2
        ]
        file-print ""
    ]
    ] [
    ask turtles with [mydad != nobody and habitat = [habitat_type] of patch-here and sample = 1] [
        let indname word self ""
        file-type (word (remove " " indname) "_" xcor "_" ycor "_" [habitat_type] of patch-here ",")
        
        let i 0
        while [i < (num_microsat * 2)] [
          let allele1 item (i + 0) (markerN)
          let allele2 item (i + 1) (markerN)
          let genotype "000000"
            
          ;allele 1
          ifelse allele1 < 10 [
            set allele1 word "00" allele1
          ] [
            if allele1 < 100 [
              set allele1 word "0" allele1
            ]
          ]
          
          ;allele 2
          ifelse allele2 < 10 [
            set allele2 word "00" allele2
          ] [
            if allele2 < 100 [
              set allele2 word "0" allele2
            ]
          ]
          
          ;genotype
          set genotype word allele1 allele2
            
          file-type word genotype " "
            
          set i i + 2
        ]
        file-print ""
    ]
    ]  
      
      
      
    set habitat habitat + 1
  ]
  file-close  
end

;-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

;---------------------------------------------------------------------------------------------------------------------##############
;---------------------------------------------------------------------------------------------------------------------##############
;   LANDSCAPE SUBMODELS
;---------------------------------------------------------------------------------------------------------------------##############
;---------------------------------------------------------------------------------------------------------------------##############

to change_habitat_type [land_m]
  let list_habitat n-values numHAB [? + 1]
  
  ifelse land_m = "none" [] [
    ifelse land_m = "random" [
      ask patches [set habitat_type one-of list_habitat]
    ] [
      ifelse land_m = "neighbor4" [
        ask patches [
          let list_habitat_neighbors [habitat_type] of neighbors4
          set habitat_type one-of list_habitat_neighbors
        ]
      ] [
        ifelse land_m = "neighbor8" [
          ask patches [
            let list_habitat_neighbors [habitat_type] of neighbors
            set habitat_type one-of list_habitat_neighbors
          ]
        ] [
          if land_m = "transition" [
            ask patches with [habitat_type > 1] [
              set habitat_type habitat_type - 1
            ]
          ]
        ]
      ]
    ]
  ]
  if land_m = "user-defined" 
  [
    ;------------------------------------------------
    ;    WRITE YOUR SCRIPT HERE
    ;------------------------------------------------    
  ]
end

;---------------------------------------------------------------------------------------------------------------------##############
;---------------------------------------------------------------------------------------------------------------------##############
;   GUI
;---------------------------------------------------------------------------------------------------------------------##############
;---------------------------------------------------------------------------------------------------------------------##############

to color_habitat_barrier
  carefully [
    let max_habitat max [habitat_barrier] of patches
    ask patches [set pcolor scale-color red habitat_barrier 0 max_habitat]
    print (word "Average landscape barrier: " mean([habitat_barrier] of patches))
  ] [ print "Landscape barriers not found." ]
end
to color_habitat_resource
  carefully [
    let max_habitat max [habitat_resource] of patches
    ask patches [set pcolor scale-color blue habitat_resource 0 max_habitat]
    print (word "Average landscape resource: " mean([habitat_resource] of patches))
  ] [ print "Landscape resources not found." ]
end
to color_habitat_type
  ask patches with [habitat_type = 1] [set pcolor 15]
  ask patches with [habitat_type = 2] [set pcolor 25]
  ask patches with [habitat_type = 3] [set pcolor 35]
  ask patches with [habitat_type = 4] [set pcolor 45]
  ask patches with [habitat_type = 5] [set pcolor 55]
  ask patches with [habitat_type = 6] [set pcolor 65]
  ask patches with [habitat_type = 7] [set pcolor 75]
  ask patches with [habitat_type = 8] [set pcolor 85]
  ask patches with [habitat_type = 9] [set pcolor 95]
  ask patches with [habitat_type = 10] [set pcolor 105]
  ask patches with [habitat_type > 10] [set pcolor 115]
  ;let max_habitat max [habitat_type] of patches
  ;ask patches [set pcolor scale-color green habitat_type 0 max_habitat]
  print (word "Number of habitat types: " max([habitat_type] of patches))
  if count(patches with [habitat_type = 0]) > 0 [print "Landscape habitat types not found."]
end

to choose_habitat_type ;script to guide the user: if one habitat is created, user can add only habitat 2 and so on
  ifelse max([habitat_type] of patches) = 0 [
    set h_choice user-one-of "choose habitat type" ["h1"] 
  ] [
    ifelse max([habitat_type] of patches) = 1 [
      set h_choice user-one-of "choose habitat type" ["h1" "h2"]
    ] [
      ifelse max([habitat_type] of patches) = 2 [
        set h_choice user-one-of "choose habitat type" ["h1" "h2" "h3"]
      ] [
        ifelse max([habitat_type] of patches) = 3 [
          set h_choice user-one-of "choose habitat type" ["h1" "h2" "h3" "h4"]
        ] [
          ifelse max([habitat_type] of patches) = 4 [
            set h_choice user-one-of "choose habitat type" ["h1" "h2" "h3" "h4" "h5"]
          ] [
            ifelse max([habitat_type] of patches) = 5 [
              set h_choice user-one-of "choose habitat type" ["h1" "h2" "h3" "h4" "h5" "h6"]
            ] [
              ifelse max([habitat_type] of patches) = 6 [
                set h_choice user-one-of "choose habitat type" ["h1" "h2" "h3" "h4" "h5" "h6" "h7"]
              ] [
                ifelse max([habitat_type] of patches) = 7 [
                  set h_choice user-one-of "choose habitat type" ["h1" "h2" "h3" "h4" "h5" "h6" "h7" "h8"]
                ] [
                  ifelse max([habitat_type] of patches) = 8 [
                    set h_choice user-one-of "choose habitat type" ["h1" "h2" "h3" "h4" "h5" "h6" "h7" "h8" "h9"]
                  ] [
                    ifelse max([habitat_type] of patches) = 9 [
                      set h_choice user-one-of "choose habitat type" ["h1" "h2" "h3" "h4" "h5" "h6" "h7" "h8" "h9" "h10"]
                    ] [
                      ifelse max([habitat_type] of patches) >= 10 [
                        set h_choice user-one-of "choose habitat type" ["h1" "h2" "h3" "h4" "h5" "h6" "h7" "h8" "h9" "h10"]
                      ] [
                        print "!"
                      ]
                    ]
                  ]
                ]
              ]
            ]
          ]
        ]
      ]
    ]
  ]
end

to draw_habitat_type
  if mouse-down? [
      ask patch mouse-xcor mouse-ycor [ 
        ifelse h_choice = "h1" [
          set habitat_type 1
          set pcolor 15
        ] [
          ifelse h_choice = "h2" [
            set habitat_type 2
            set pcolor 25
          ] [
            ifelse h_choice = "h3" [
              set habitat_type 3
              set pcolor 35
            ] [
              ifelse h_choice = "h4" [
                set habitat_type 4
                set pcolor 45
              ] [
                ifelse h_choice = "h5" [
                  set habitat_type 5
                  set pcolor 55
                ] [
                  ifelse h_choice = "h6" [
                    set habitat_type 6
                    set pcolor 65
                  ] [
                    ifelse h_choice = "h7" [
                      set habitat_type 7
                      set pcolor 75
                    ] [
                      ifelse h_choice = "h8" [
                        set habitat_type 8
                        set pcolor 85
                      ] [
                        ifelse h_choice = "h9" [
                          set habitat_type 9
                          set pcolor 95
                        ] [
                          if h_choice = "h10" [
                            set habitat_type 10
                            set pcolor 105
                          ]
                        ]
                      ]
                    ]
                  ]
                ]
              ]
            ]
          ]
        ]
        set habitat_barrier h_barrier
        set habitat_resource h_resource
      ]
    ]
end     
     
@#$#@#$#@
GRAPHICS-WINDOW
470
50
715
130
-1
-1
9.8
1
10
1
1
1
0
0
0
1
0
19
0
4
0
0
1
ticks
30.0

PLOT
320
585
675
805
population dynamics
Time (in generations)
# of individuals
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "plot count turtles"

SLIDER
320
285
675
318
var_num_generations
var_num_generations
1
100000
100
1
1
NIL
HORIZONTAL

INPUTBOX
15
10
460
70
wd
NIL
1
0
String

INPUTBOX
15
95
155
155
habitat_barrier_file
habitat_barrier.txt
1
0
String

INPUTBOX
15
156
155
216
habitat_resource_file
habitat_resource.txt
1
0
String

INPUTBOX
15
215
155
275
habitat_type_file
habitat_type.txt
1
0
String

SLIDER
15
465
160
498
num_microsat
num_microsat
0
100
10
1
1
NIL
HORIZONTAL

SLIDER
218
430
310
463
sd_H
sd_H
0
5
1
0.1
1
NIL
HORIZONTAL

SLIDER
15
575
310
608
num_locus_per_habitat
num_locus_per_habitat
1
10
1
1
1
NIL
HORIZONTAL

SLIDER
15
430
120
463
nb_agent
nb_agent
1
100
100
1
1
NIL
HORIZONTAL

SLIDER
165
500
310
533
r_growth
r_growth
0
1
0.5
0.01
1
NIL
HORIZONTAL

SLIDER
15
535
310
568
nb_move
nb_move
1
10
1
1
1
NIL
HORIZONTAL

SLIDER
15
610
160
643
var_coef_selec_vs_xx
var_coef_selec_vs_xx
0.01
1
0.5
0.01
1
NIL
HORIZONTAL

SLIDER
165
610
310
643
var_dom_degree_a
var_dom_degree_a
0.01
1
0.5
0.01
1
NIL
HORIZONTAL

SLIDER
15
500
160
533
proba_dispersion
proba_dispersion
0
1
0.5
0.001
1
NIL
HORIZONTAL

SLIDER
165
465
310
498
mutation_rateN
mutation_rateN
0
1
1.0E-4
0.000001
1
NIL
HORIZONTAL

INPUTBOX
65
655
115
715
in_X
10
1
0
Number

INPUTBOX
115
655
165
715
in_Y
4
1
0
Number

TEXTBOX
30
75
135
93
landscape from files:
11
0.0
1

TEXTBOX
180
75
300
93
landscape from the GUI:
11
0.0
1

INPUTBOX
170
95
240
155
land_height
3
1
0
Number

INPUTBOX
240
95
310
155
land_length
3
1
0
Number

BUTTON
170
160
310
193
APPLY NEW SCALE
setup\nresize-world 0 (land_length - 1) 0 (land_height - 1)
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
170
195
310
228
START DRAWING
display\ndraw_habitat_type
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
170
230
310
263
CHOOSE HABITAT TYPE
choose_habitat_type
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
170
265
310
310
You are currently drawing:
h_choice
17
1
11

INPUTBOX
170
310
240
370
h_resource
100
1
0
Number

INPUTBOX
240
310
310
370
h_barrier
10
1
0
Number

TEXTBOX
325
75
450
93
landscape from example:
11
0.0
1

BUTTON
15
385
160
418
[1] LOAD FILES (SETUP)
setup\nload_input_files
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
165
385
310
418
[1] LOAD GUI (SETUP)
load_input_GUI\nexport_GUI\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
320
95
465
151
RUN EXAMPLE C1: RANDOM
setup\nload_input_example1
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SWITCH
190
650
310
683
everywhere
everywhere
1
1
-1000

TEXTBOX
20
670
65
700
initial location
11
0.0
1

TEXTBOX
175
660
190
678
or
11
0.0
1

CHOOSER
15
725
310
770
land_management
land_management
"none" "random" "neighbor4" "neighbor8" "transition" "user-defined"
0

SLIDER
15
770
310
803
land_time_step
land_time_step
0
1000
1
1
1
NIL
HORIZONTAL

SLIDER
320
355
675
388
output_freq
output_freq
1
var_num_generations
100
1
1
NIL
HORIZONTAL

SWITCH
320
395
425
428
CSV
CSV
0
1
-1000

SWITCH
320
430
425
463
GENELAND
GENELAND
0
1
-1000

SWITCH
570
430
675
463
ARLEQUIN
ARLEQUIN
0
1
-1000

SWITCH
445
430
550
463
GENEPOP
GENEPOP
0
1
-1000

SWITCH
570
395
675
428
FSTAT
FSTAT
0
1
-1000

SWITCH
445
395
550
428
STRUCTURE
STRUCTURE
0
1
-1000

INPUTBOX
550
470
610
530
n_ind
25
1
0
Number

INPUTBOX
615
470
675
530
n_points
20
1
0
Number

SWITCH
320
470
445
503
sample_pop?
sample_pop?
0
1
-1000

BUTTON
320
535
675
576
[2] RUN SIMULATION (GO)
ifelse count(patches with [habitat_type = 0]) = 0 [\n  repeat var_num_repetitions [\n    setup\n    load_input_files\n    setup_create_turtles\n    go_init\n  ]\n] [\n  print \"Missing values have been found in input files. Simulation stopped\"\n]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
320
155
465
206
RUN EXAMPLE C2: BLOCS
setup\nload_input_example2
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
320
210
465
261
RUN EXAMPLE C3: BOXES
setup\nload_input_example3
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

INPUTBOX
15
275
155
335
FSTAT_file
myfstat.dat
1
0
String

TEXTBOX
175
695
190
713
or
11
0.0
1

SWITCH
190
685
310
718
from_fstat_file
from_fstat_file
1
1
-1000

CHOOSER
450
470
542
515
SRS
SRS
"WR" "WOR"
0

SLIDER
320
320
675
353
var_num_repetitions
var_num_repetitions
0
1000
1
1
1
NIL
HORIZONTAL

BUTTON
470
10
555
43
view updates
display
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SWITCH
125
430
215
463
biall?
biall?
1
1
-1000

@#$#@#$#@
## CREDITS AND REFERENCES

Copyright (C) 2012 Francois Rebaudo (1) 

(1) IRD, UR 072, CNRS, UPR 9034, LEGS 91198 Gif-sur-Yvette Cedex and Universite Paris-Sud 11, 91405 Orsay Cedex, France
francois.rebaudo@ird.fr

>This program is free: you can redistribute the code and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>. 

>Scientists please be aware that the fact that this program is released as Free Software does not excuse you from scientiﬁc propriety,which obligates you to give appropriate credit. If you write a scientiﬁc paper describing research that made substantive use of this program, it is your obligation as a scientist to mention the fashion in which this model was used, including the version number, with a citation to the literature, in the Methods section.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -7500403 true true 135 285 195 285 270 90 30 90 105 285
Polygon -7500403 true true 270 90 225 15 180 90
Polygon -7500403 true true 30 90 75 15 120 90
Circle -1 true false 183 138 24
Circle -1 true false 93 138 24

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="example_C1_RANDOM" repetitions="100" runMetricsEveryStep="false">
    <setup>setup
load_input_example1</setup>
    <timeLimit steps="100"/>
    <enumeratedValueSet variable="wd">
      <value value="&quot;&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="habitat_resource_file">
      <value value="&quot;habitat_resource.txt&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="habitat_barrier_file">
      <value value="&quot;habitat_barrier.txt&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="habitat_type_file">
      <value value="&quot;habitat_type.txt&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="CSV">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="GENEPOP">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="FSTAT">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="STRUCTURE">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="GENELAND">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ARLEQUIN">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="output_freq">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sample_pop?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="n_points">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="n_ind">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nb_agent">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="in_Y">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="in_X">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="everywhere">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="r_growth">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nb_move">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="proba_dispersion">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num_microsat">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation_rateN">
      <value value="1.0E-4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sd_H">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num_locus_per_habitat">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="var_coef_selec_vs_xx">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="var_dom_degree_a">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_management">
      <value value="&quot;none&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_time_step">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_length">
      <value value="40"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_height">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="h_barrier">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="h_resource">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="var_num_generations">
      <value value="100"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="example_C2_BLOCS" repetitions="100" runMetricsEveryStep="false">
    <setup>setup
load_input_example2</setup>
    <timeLimit steps="100"/>
    <enumeratedValueSet variable="wd">
      <value value="&quot;&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="habitat_resource_file">
      <value value="&quot;habitat_resource.txt&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="habitat_barrier_file">
      <value value="&quot;habitat_barrier.txt&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="habitat_type_file">
      <value value="&quot;habitat_type.txt&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="CSV">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="GENEPOP">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="FSTAT">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="STRUCTURE">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="GENELAND">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ARLEQUIN">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="output_freq">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sample_pop?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="n_points">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="n_ind">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nb_agent">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="in_Y">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="in_X">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="everywhere">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="r_growth">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nb_move">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="proba_dispersion">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num_microsat">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation_rateN">
      <value value="1.0E-4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sd_H">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num_locus_per_habitat">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="var_coef_selec_vs_xx">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="var_dom_degree_a">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_management">
      <value value="&quot;none&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_time_step">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_length">
      <value value="40"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_height">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="h_barrier">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="h_resource">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="var_num_generations">
      <value value="100"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="example_C3_BOXES" repetitions="100" runMetricsEveryStep="false">
    <setup>setup
load_input_example3</setup>
    <timeLimit steps="100"/>
    <enumeratedValueSet variable="wd">
      <value value="&quot;&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="habitat_resource_file">
      <value value="&quot;habitat_resource.txt&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="habitat_barrier_file">
      <value value="&quot;habitat_barrier.txt&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="habitat_type_file">
      <value value="&quot;habitat_type.txt&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="CSV">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="GENEPOP">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="FSTAT">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="STRUCTURE">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="GENELAND">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ARLEQUIN">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="output_freq">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sample_pop?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="n_points">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="n_ind">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nb_agent">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="in_Y">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="in_X">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="everywhere">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="r_growth">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nb_move">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="proba_dispersion">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num_microsat">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation_rateN">
      <value value="1.0E-4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sd_H">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num_locus_per_habitat">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="var_coef_selec_vs_xx">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="var_dom_degree_a">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_management">
      <value value="&quot;none&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_time_step">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_length">
      <value value="40"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_height">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="h_barrier">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="h_resource">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="var_num_generations">
      <value value="100"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="test" repetitions="2" runMetricsEveryStep="true">
    <setup>setup
load_input_files
setup_create_turtles
go_init</setup>
    <timeLimit steps="100"/>
    <enumeratedValueSet variable="FSTAT_file">
      <value value="&quot;myfstat.dat&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="CSV">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="n_points">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num_microsat">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="GENELAND">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="wd">
      <value value="&quot;&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="GENEPOP">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="var_coef_selec_vs_xx">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation_rateN">
      <value value="1.0E-4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="h_barrier">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="habitat_type_file">
      <value value="&quot;my_habitat_type.txt&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ARLEQUIN">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sd_H">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="var_num_generations">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_length">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="output_freq">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="STRUCTURE">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_time_step">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="var_num_repetitions">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sample_pop?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="habitat_barrier_file">
      <value value="&quot;my_habitat_barrier.txt&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nb_move">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="n_ind">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="var_dom_degree_a">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="FSTAT">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_height">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nb_agent">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="in_X">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="r_growth">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="in_Y">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="habitat_resource_file">
      <value value="&quot;my_habitat_resource.txt&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="everywhere">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="biall?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="from_fstat_file">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="land_management">
      <value value="&quot;transition&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num_locus_per_habitat">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="h_resource">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="SRS">
      <value value="&quot;WR&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="proba_dispersion">
      <value value="0.5"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
1
@#$#@#$#@
